@@ -157,6 +157,7 @@ sub update ($self, %args) {
         # prepare to be able to put back data matching an existing dir
         if ($old_path eq '*' or ($old_path =~ m!(.*)/\*$! and $current_dir->child($1)->is_dir )) {
             $preserved_path{$old_path} = delete $old_split_files{$old_path};
+            $self->_say( "Note: preserving '$old_path'" );
         }
         else {
             $self->_say( "Note: '$old_path' was removed (or excluded) from new upstream source" );
@@ -239,7 +240,7 @@ sub update ($self, %args) {
         # if full_license is not provided in datum, check global license(s)
         if (not $datum->{License}{full_license}) {
             my $ok = 0;
-            my @sub_licenses = split /[,\s]+(?:or|and)[,\s]+/,$l;
+            my @sub_licenses = split m![,\s]+(?:and/or|or|and)[,\s]+!,$l;
             my $lic_count = 0;
             my @empty_licenses = grep {
                 my $text = $self->grab_value(steps => qq!License:"$_" text!, check =>'no') ;
@@ -289,9 +290,15 @@ sub update ($self, %args) {
         $files_obj->fetch_with_id($old_path)->load_data( $preserved_path{$old_path} );
     }
 
-    # put back debian data
+    # put back debian data if file or dir is still present
     foreach my $deb_path (sort keys %debian_paths) {
-        $files_obj->fetch_with_id($deb_path)->load_data( $debian_paths{$deb_path} );
+        my $target = $deb_path =~ m!(.*)/\*$! ? $1 : $deb_path;
+        if ($current_dir->child($target)->exists) {
+            $files_obj->fetch_with_id($deb_path)->load_data( $debian_paths{$deb_path} );
+        }
+        else {
+            $self->_say("Note: dropping obsolete entry $deb_path");
+        }
     }
 
     $self->_apply_fix_scan_copyright_file($current_dir) ;
@@ -388,6 +395,15 @@ In debian package directory:
 
 Note: this command is experimental.
 
+=head1 Debian copyright data
+
+The C<Files: debian/*> section from C<debian/copyright> is often the
+only place containing copyright information for the files created by
+Debian maintainer. So all C<Files> entries beginning with C<debian/>
+are preserved during update. However, entries not matching an existing
+file or directory are removed.
+
+
 =head1 Tweak results
 
 Results can be tweaked either by:
@@ -423,8 +439,14 @@ based on comments, the result is sometimes lackluster. Your may
 specify instruction to alter or set specific copyright entries in
 C<debian/fix.scanned.copyright> file
 (or C<< debian/<source-package>.fix.scanned.copyright >>).
-Each line of this file will be handled
-by L<Config::Model::Loader> to modify copyright information.
+
+L<cme> stores the copyright information in a tree. Entries in
+C<fix.scanned.copyright> provide instructions for traversing the cme tree
+and modifying entries. You can have a view of C<debian/copyright> file
+translated in this syntax by running C<cme dump --format cml
+dpkg-copyright>.  Each line of this file will be handled by
+L<Config::Model::Loader> to modify copyright information; the full
+syntax is documented in L<Config::Model::Loader/"load string syntax"> section.
 
 =head2 Example
 
@@ -464,6 +486,12 @@ Here's another more complex example:
  # and modify the copyright entry with a Perl substitution
  ! Files:~/^3rdparty/ Copyright=~s/@/(at)/
 
+Sometimes, you might want to find an entry that spans multiple lines.
+You can do this by double quoting the whole value:
+
+ ! Files:"uulib/crc32.h
+ uulib/uustring.h" Copyright="2019 John Doe"
+
 =head1 Under the hood
 
 This section explains how cme merges the information from the existing
