@@ -35,8 +35,8 @@ my $madison_host = 'api.ftp-master.debian.org';
 my $madison_endpoint = "https://$madison_host/madison";
 
 # list of virtual packages
-# See https://www.debian.org/doc/packaging-manuals/virtual-package-names-list.txt
-# updated from 20 Feb 2019 version
+# See https://www.debian.org/doc/packaging-manuals/virtual-package-names-list.yaml
+# updated from 16 May 2020 version
 my @virtual_list = qw/
  adventure
  audio-mixer
@@ -46,7 +46,10 @@ my @virtual_list = qw/
  c-compiler
  c-shell
  cron-daemon
+ dbus-session-bus
  debconf-2.0
+ default-dbus-session-bus
+ default-logind
  dhcp-client
  dict-client
  dict-server
@@ -88,6 +91,7 @@ my @virtual_list = qw/
  lambdamoo-server
  libc-dev
  linux-kernel-log-daemon
+ logind
  lzh-archiver
  mail-reader
  mail-transport-agent
@@ -140,15 +144,43 @@ my @virtual_list = qw/
 # other less official virtual packages
 push @virtual_list, qw/
  debhelper-compat
+ libkvm-dev
  libgl-dev
  libqt5opengl5-desktop-dev
  libtiff-dev
  libuv-dev
  ruby-interpreter
+ perl-xs-dev
  ssh-client
  ssh-server
 /;
 
+# debhelper sequences
+# grep-available -F Provides dh-sequence- -s Provides -n | grep -Eo 'dh-sequence-[-a-z]+' | sort -u | xargs printf " %s\n"
+push @virtual_list, qw/
+ dh-sequence-ada-library
+ dh-sequence-cli
+ dh-sequence-cme-upgrade
+ dh-sequence-dist-zilla
+ dh-sequence-dwz
+ dh-sequence-elf-tools
+ dh-sequence-fortran-mod
+ dh-sequence-gir
+ dh-sequence-gnome
+ dh-sequence-installinitramfs
+ dh-sequence-libva
+ dh-sequence-numpy
+ dh-sequence-perl-dbi
+ dh-sequence-perl-imager
+ dh-sequence-perl-openssl
+ dh-sequence-pypy
+ dh-sequence-python
+ dh-sequence-scour
+ dh-sequence-sphinxdoc
+ dh-sequence-systemd
+ dh-sequence-zathura
+/;
+
 my %virtual_hash = map {( $_ => 1); } @virtual_list;
 
 my %debian_map;
@@ -297,8 +329,24 @@ eofile: /^\Z/
 
 EOG
 
+# extract installed version of debhelper
+my $str = `/usr/bin/dpkg -l debhelper`;
+my ($current_compat_value) = ($str =~ /ii\s+debhelper\s+(\d+)/);
+
+if (not $current_compat_value) {
+    die "debhelper package is not installed";
+}
+
 my $parser ;
 
+after 'store' => sub {
+    my $self = shift;
+    my $data = $self->_fetch_no_check;
+    if ( $data =~ /^debhelper-compat\s*\(\s*=\s*(\d+)\s*\)/ ) {
+        $self->parent->fetch_element(accept_hidden => 1, name => "debhelper-version")->store($1);
+    }
+};
+
 sub dep_parser {
     $parser ||= Parse::RecDescent->new($grammar) ;
     return $parser ;
@@ -325,6 +373,8 @@ sub check_dependency {
     my ($value, $check, $silent, $notify_change, $ok, $apply_fix)
         = @args{qw/value check silent notify_change ok fix/} ;
 
+    return ($ok, $value) if defined $check and $check eq 'no';
+
     # value is one dependency, something like "perl ( >= 1.508 )"
     # or exim | mail-transport-agent or gnumach-dev [hurd-i386]
 
@@ -351,7 +401,7 @@ sub check_dependency {
     my @msgs;
 
     foreach my $dep (@dep_chain) {
-        next unless ref($dep) ; # no need to check variables
+        next unless ref($dep) ; # no need to check debhelper variables
         $self->check_or_fix_pkg_name($apply_fix, $dep, $old, \@msgs) ;
 		$self->check_or_fix_essential_package($apply_fix, $dep, \@msgs) ;
 		$self->check_or_fix_dep($apply_fix, $dep, $old, \@msgs) ;
@@ -377,15 +427,69 @@ sub check_dependency {
     return ($ok, $new) ;
 }
 
-sub check_debhelper_version {
+sub check_debhelper_compat_version {
     my ($self, $apply_fix, $dep_info, $msgs) = @_ ;
-    my $dep_name = $dep_info->{name};
     my ($oper, $dep_v) = @{ $dep_info->{dep} || []};
 
-    my $dep_string = $self->struct_to_dep($dep_info) ;
-    my $lintian_dep = Lintian::Relation->new( $dep_string ) ;
-    $logger->debug("checking '$dep_string' with lintian");
+    if ( $self->check_compat_object() ) {
+        $self->check_compat_object_value($apply_fix, $msgs);
+    }
 
+    # use the compat value mentioned in debhelper-compat dependency
+    my $compat_value = defined $dep_v ? ($dep_v =~ s/~$//r) : undef;
+
+    if (not defined $compat_value or $compat_value < $current_compat_value - 1 ) {
+        if ($apply_fix) {
+            $dep_info->{dep} = [ '=' , $current_compat_value ];
+            my $msg = "changed debhelper-compat version dependency to $current_compat_value";
+            $logger->info($msg);
+            push $msgs->@*, $msg;
+        }
+        else {
+            $self->{nb_of_fixes}++ ;
+            my $str = defined $compat_value ? "too old ($compat_value)" : 'undefined';
+            my $msg = "debhelper-compat dependency version is $str. It should be $current_compat_value";
+            $self->add_warning( $msg );
+            $logger->info("will warn: $msg (fix++)");
+        }
+    }
+}
+
+sub check_debhelper_version {
+    my ($self, $apply_fix, $dep_info, $msgs) = @_ ;
+    my ($oper, $dep_v) = @{ $dep_info->{dep} || []};
+
+    return unless $self->check_compat_object();
+
+    $self->check_compat_object_value($apply_fix, $msgs);
+
+    # use the compat value mentioned in debhelper dependency
+    my $original_compat_value = defined $dep_v ? ($dep_v =~ s/~$//r) : undef;
+
+    # check if this value is current
+    my $compat_value = $self->check_compat_value( $original_compat_value, $apply_fix, $msgs );
+
+    # transform to new debhelper-compat dependency. We need to check
+    # if it's necessary because this method is called by apply_fixes
+    # until no fix can be applied.
+    if ($dep_info->{name} eq 'debhelper') {
+        if ($apply_fix) {
+            $dep_info->{name} = 'debhelper-compat';
+            $dep_info->{dep} = [ '=' , $compat_value ];
+            my $msg = "changed debhelper dependency to debhelper dependency";
+            $logger->info($msg);
+            push $msgs->@*, $msg;
+        }
+        else {
+            $self->{nb_of_fixes}++ ;
+            my $msg = "debhelper dependency is deprecated. It should be a dependency for debhelper-compat package";
+            $self->add_warning( $msg );
+            $logger->info("will warn: $msg (fix++)");
+        }
+    }
+}
+
+sub check_compat_object ($self) {
     # try to create compat_obj, but do not try twice (hence the exists test)
     # compat_obj is undef when running with 'cme edit dpkg-control'
     if (not exists $self->{_compat_obj} ) {
@@ -394,35 +498,46 @@ sub check_debhelper_version {
         my $c = $self->{_compat_obj} = $self->grab(mode => 'loose', step => "!Dpkg compat") ;
         $c->register_dependency($self) if defined $c;
     }
+    return $self->{_compat_obj} ;
+}
 
-    return unless defined $self->{_compat_obj};
-
+sub check_compat_object_value ($self, $apply_fix, $msgs) {
     my $compat_value = $self->{_compat_obj}->fetch;
 
-    # lintian does not accept space after the version number
-    my $min_dep = Lintian::Relation->new("debhelper ( >= $compat_value~)") ;
-
-    if ($lintian_dep->implies ($min_dep)) {
-        $logger->debug($dep_string." -> ".$lintian_dep->unparse." implies ". $min_dep->unparse);
-        return;
+    if (defined $compat_value) {
+        if ($apply_fix) {
+            $self->{_compat_obj}->clear;
+            $logger->info("Cleared deprecated compat value");
+            push $msgs->@*, "Cleared deprecated compat value";
+        }
+        else {
+            $self->{nb_of_fixes}++ ;
+            my $msg = "compat parameter is deprecated. "
+                . "Please use debhelper-compat dependency. See debhelper(7) for details.";
+            $self->add_warning( $msg );
+            $logger->info("will warn: $msg (fix++)");
+        }
     }
+}
 
-    $logger->debug("'$dep_string' does not imply debhelper >= $compat_value~");
+sub check_compat_value ($self, $compat_value, $apply_fix, $msgs) {
 
-    # $show_rel avoids undef warnings
-    my $show_rel = join(' ', map { $_ || ''} ($oper, $dep_v));
-    my $msg = "should be (>= $compat_value~) not ($show_rel) because compat is $compat_value" ;
-    if ($apply_fix) {
-        $dep_info->{dep} = [ '>=', $compat_value.'~' ] ; # notify_change called in check_value
-        $logger->info("fixed debhelper dependency from "
-            ."$dep_name $show_rel -> ".$min_dep->unparse." (for compat $compat_value)");
-        push $msgs->@*, $msg;
-    }
-    else {
-        $self->{nb_of_fixes}++ ;
-        $self->add_warning( $msg );
-        $logger->info("will warn: $msg (fix++)");
+    if (not defined $compat_value or $compat_value < $current_compat_value - 1 ) {
+        if ($apply_fix) {
+            my $msg = "Set debhelper dependency version to $current_compat_value~";
+            $logger->info($msg);
+            push $msgs->@*, $msg;
+            $compat_value = $current_compat_value;
+        }
+        else {
+            $self->{nb_of_fixes}++ ;
+            my $str = defined $compat_value ? "too old ($compat_value)" : 'undefined';
+            my $msg = "debhelper dependency version is $str. It should be $current_compat_value";
+            $self->add_warning( $msg );
+            $logger->info("will warn: $msg (fix++)");
+        }
     }
+    return $compat_value;
 }
 
 sub struct_to_dep {
@@ -679,54 +794,51 @@ sub check_or_fix_dep {
         return;
     }
 
-    if ( $pkg eq 'debhelper' ) {
-        return $self->check_debhelper_version( $apply_fix, $dep_info, $msgs );
+    # mess with debhelper compat only in Build dependencies
+    if ( $self->element_name ne 'Depends' ) {
+        if ($pkg eq 'debhelper' ) {
+            return $self->check_debhelper_version( $apply_fix, $dep_info, $msgs );
+        }
+
+        if ( $pkg eq 'debhelper-compat' ) {
+            return $self->check_debhelper_compat_version( $apply_fix, $dep_info, $msgs );
+        }
     }
 
     my ( $vers_dep_ok, @list ) =  $self->check_versioned_dep( $dep_info );
     return if $vers_dep_ok;
 
+    my ($warn_str, $log_str, $fix_sub);
     if ($dep_info->{dep}[0] =~ /</) {
-        $self->warn_or_remove_dep ($apply_fix, $dep_info, \@list, $msgs) ;
+        $warn_str = "unnecessary older-than versioned dependency: ". $self-> struct_to_dep($dep_info)
+            . ". Debian has @list";
+        $log_str = "removed dependency $dep_info->{name}";
+        # the fix removes the whole dependency
+        $fix_sub =  sub{ $_[0]->%* = () };
     }
     else {
-        $self->warn_or_remove_vers_dep ($apply_fix, $dep_info, \@list, $msgs) ;
+        $warn_str = "unnecessary greater-than versioned dependency: ". $self-> struct_to_dep($dep_info)
+            . ". Debian has @list";
+        $log_str = "removed greater-than versioned dependency from $dep_info->{name}";
+        # the fix removes greater-than versioned dep, notify_change is called in check_value
+        $fix_sub = sub { delete $_[0]->{dep} };
     }
+    $self->warn_or_fix_dep_info ($fix_sub, $apply_fix, $dep_info, $msgs, $warn_str, $log_str) ;
 }
 
-sub warn_or_remove_vers_dep {
-    my ( $self, $apply_fix, $dep_info, $list, $msgs ) = @_;
-    my $pkg = $dep_info->{name};
 
-    my $msg = "unnecessary greater-than versioned dependency: ". $self-> struct_to_dep($dep_info)
-        . ". Debian has @$list";
+sub warn_or_fix_dep_info {
+    my ( $self, $fix_sub, $apply_fix, $dep_info, $msgs, $warn_msg, $log_str ) = @_;
+
     if ($apply_fix) {
-        delete $dep_info->{dep};    # remove greater-than versioned dep, notify_change called in check_value
-        $logger->info("fix: removed greater-than versioned dependency from $pkg");
-        push $msgs->@*, $msg;
+        $fix_sub->($dep_info);
+        $logger->info("fix: $log_str");
+        push $msgs->@*, $warn_msg;
     }
     else {
         $self->{nb_of_fixes}++;
-        $self->add_warning( $msg);
-        $logger->info("will warn: $msg (fix++)");
-    }
-}
-
-sub warn_or_remove_dep {
-    my ( $self, $apply_fix, $dep_info, $list, $msgs ) = @_;
-    my $pkg = $dep_info->{name};
-
-    my $msg = "unnecessary older-than versioned dependency: ". $self-> struct_to_dep($dep_info)
-        . ". Debian has @$list";
-    if ($apply_fix) {
-        %$dep_info = ();    # remove whole dependency
-        $logger->info("fix: removed dependency $pkg");
-        push $msgs->@*, $msg;
-    }
-    else {
-        $self->{nb_of_fixes}++;
-        $self->add_warning( $msg);
-        $logger->info("will warn: $msg (fix++)");
+        $self->add_warning( $warn_msg );
+        $logger->info("will warn: $warn_msg (fix++)");
     }
 }
 
@@ -787,17 +899,20 @@ sub get_available_version {
 
     my $url = "$madison_endpoint?package=".uri_escape($pkg_name).'&f&b=deb' ;
     $self->instance->show_message("Connecting to $madison_host to check $pkg_name versions. Please wait...") ;
-	my $body = get($url);
+	my $body = get($url); # returns a json list
     my $res = [];
 	if (defined $body) {
         my $ref = extract_madison_info($body);
-        $self->instance->show_message("got info for $pkg_name") ;
+        my $msg = $ref->{$pkg_name} ? 'info' : 'no info';
+        $self->instance->show_message("got $msg for $pkg_name") ;
         $res = $ref->{$pkg_name} || [];
         $logger->debug("pkg info is @$res");
+        # unknown package return an empty body, let's not retry twice the same unknown package.
+        $tried{$pkg_name} = 1;
     }
     else {
+        # HTTP error
         warn "cannot get data for package $pkg_name from $madison_endpoint.\n" unless defined $body ;
-        $tried{$pkg_name} = 1;
     }
 
 	return $res->@*;
