@@ -1,4 +1,7 @@
-[
+use strict;
+use warnings;
+
+return [
   {
     'author' => [
       'Dominique Dumont'
@@ -16,16 +19,9 @@
       },
       'compat',
       {
-        'class' => 'Config::Model::Dpkg::Compat',
         'description' => 'compat file defines the debhelper compatibility level',
         'type' => 'leaf',
-        'value_type' => 'integer',
-        'warn_if' => {
-          'compat value is too old' => {
-            'code' => '$_ < $self->_fetch_std - 1',
-            'fix' => '$_ = $self->_fetch_std'
-          }
-        }
+        'value_type' => 'integer'
       },
       'control',
       {
@@ -65,7 +61,7 @@ include /usr/share/dpkg/default.mk
       {
         'compute' => {
           'allow_override' => '1',
-          'formula' => '$pkg_name (0.001-1) UNRELEASED; urgency=medium
+          'formula' => '$pkg_name (0.0.0.0-0) UNRELEASED; urgency=medium
 
   * ...
 
@@ -125,9 +121,27 @@ details',
           'config_class_name' => 'Dpkg::Install',
           'type' => 'node'
         },
-        'description' => 'List the files to install into each package and the directory they should be installed to.',
+        'description' => 'List the files to install into each package and the directory they should be installed to.
+
+Here the mapping between the install files and the install key:
+
+'.'=over 4
+
+'.'=item *
+
+"." -> C<debian/install>
+
+'.'=item *
+
+"package" -> C<debian/package.install>
+
+'.'=item *
+
+"package/arch" -> C<debian/package.install.arch>
+
+'.'=back
+',
         'index_type' => 'string',
-        'migrate_keys_from' => '- control binary',
         'type' => 'hash'
       },
       'not-installed',
@@ -180,7 +194,6 @@ Please keep in mind that dh_install will not expand wildcards in this file.
           'config_class_name' => 'Dpkg::PackageScripts',
           'type' => 'node'
         },
-        'follow_keys_from' => '- control binary',
         'index_type' => 'string',
         'type' => 'hash'
       },
