@@ -1,8 +1,11 @@
-[
+use strict;
+use warnings;
+
+return [
   {
     'class_description' => 'Contains the list of files to be installed by L<dh_install>',
     'element' => [
-      'list',
+      'content',
       {
         'cargo' => {
           'type' => 'leaf',
@@ -19,7 +22,7 @@ Note that if you list exactly one filename or wildcard-pattern, with
 no explicit destination, then dh_install will automatically guess the
 destination to use, the same as if the --autodest option were used.
 
-See L<dh_install> for more details.
+See L<debhelper(7)> and L<dh_install> for more details.
 ',
         'duplicates' => 'warn',
         'type' => 'list'
@@ -29,9 +32,9 @@ See L<dh_install> for more details.
     'rw_config' => {
       'auto_create' => '1',
       'auto_delete' => '1',
-      'backend' => 'PlainFile',
+      'backend' => 'Dpkg::DebHelperFile',
       'config_dir' => 'debian',
-      'file' => '&index(-).install'
+      'file' => 'install'
     }
   }
 ]
