@@ -1,25 +1,27 @@
+use strict;
+use warnings;
+
 use Data::Dumper;
-use IO::File;
-use File::HomeDir ;
+use Path::Tiny;
 use Carp;
 use Carp::Assert;
 use Carp::Assert::More;
-use strict;
-use warnings;
 use utf8;
-use Config::Model::Dpkg::Compat;
+use Config::Model::BackendMgr ;
+use Config::Model::Dpkg::Dependency;
 
-our $conf_file_name = "";
-our $conf_dir       = '';
-our $model_to_test  = "Dpkg";
+my $conf_file_name = "";
+my $conf_dir       = '';
+my $model_to_test  = "Dpkg";
 
 eval { require AptPkg::Config; };
-our $skip = ( $@ or not -r '/etc/debian_version' ) ? 1 : 0;
+my $skip = ( $@ or not -r '/etc/debian_version' ) ? 1 : 0;
 
-our $home_for_test = '/home/joe' ;
+my $home_for_test = '/home/joe' ;
 Config::Model::BackendMgr::_set_test_home($home_for_test) ;
 
-my $current_compat = Config::Model::Dpkg::Compat::_fetch_std;
+my $str = `/usr/bin/dpkg -l debhelper`;
+my ($current_compat) = ($str =~ /ii\s+debhelper\s+(\d+)/);
 
 # required to set value that enable creation of dummy changelog:
 # changelog is computed from my_config email and name which are also
@@ -37,6 +39,7 @@ my %tweak_map = (
     config => "$home_for_test/.config/config-model/dpkg-meta.yml",
     log => '/debian/changelog',
     rules => '/debian/rules',
+    '-compat' => '/debian/compat',
 );
 
 # returns a sub used to tweak list of debian files
@@ -72,22 +75,26 @@ sub tweak {
     } "old home dir is gone";
 }
 
-@tests = (
+my @tests = (
     {   name => 't0',
         check => {
             'control source Build-Depends-Indep:3' => 'libtest-pod-perl',
             # there's no apply_fix in this test, that's why compat and debhelper stay at 8
             'control source Build-Depends:0' => 'debhelper (>= 8)',
             'compat' => 8,
+            'package-scripts:. postinst' => qr/dummy postinst/,
+            'package-scripts:t0 prerm' => qr/dummy prerm/,
+            'package-scripts:t0/amd64 postrm' => qr/dummy postrm script for amd64/,
+            'package-scripts:./amd64 preinst' => qr/dummy preinst script for amd64/,
         },
         log4perl_load_warnings => [[
-            User =>  map {(warn => $_)} qr/compat/, qr/source Standards-Version/, qr/Dual dependency/
+            User =>  map {(warn => $_)} qr/source Standards-Version/, qr/compat/, (qr/debhelper/) x 2 , qr/Dual dependency/
         ]],
     },
 
     {   name => 't1',
         log4perl_load_warnings => [[
-            User => map {(warn => $_)} qr/compat/, qr/standard/, qr/compat/, qr/canonical/, qr/invalid/
+            User => map {(warn => $_)} qr/standard/, qr/compat/, (qr/debhelper/) x 2 , qr/canonical/, qr/invalid/
         ]],
         apply_fix => 1 ,
         load => qq!patches:fix-spelling Description="more spelling details"!
@@ -101,9 +108,10 @@ sub tweak {
             'patches:glib-single-include Synopsis', "mega patchchoid",
             'patches:use-standard-dzil-test Description',
               "Test is modified in order not to load the Test:Dzil module\nprovided in t/lib",
-            'compat' => $current_compat , # test setup and fix
-            'control source Build-Depends:0' => "debhelper (>= $current_compat~)" ,
-        },warnings => [ (qr/deprecated/) x 3 ],
+            'control source Build-Depends:0' => "debhelper-compat (= $current_compat)" ,
+        },
+        file_check_sub => tweak(qw!-compat!),
+        warnings => [ (qr/deprecated/) x 3 ],
     },
 
     {
@@ -128,6 +136,7 @@ sub tweak {
             'bugfiles:libversion bug-control report-with' => 'libreoffice-core',
             'bugfiles:libversion bug-control package-status' => 'udev dracut initramfs-tools',
         },
+        file_check_sub => tweak(qw!-compat!),
     },
     {
         name => 'pan-copyright-from-scratch',
@@ -273,17 +282,28 @@ sub tweak {
         check => {
             'copyright Files:"modules/Perl6-MIME-Base64/*" Copyright' => 'Adrian White',
             'copyright Files:"modules/DBIish/*" Comment' => qr/should be preserved by cme update/,
-            'install:rakudo-star list:0' => 'usr/bin/*',
-            'install:rakudo-star list:1' => 'usr/share/*',
+            'install:rakudo-star content:0' => 'usr/bin/*',
+            'install:rakudo-star content:1' => 'usr/share/*',
+            'install:rakudo-star/dummy content:0' => 'usr/share/dummy/*',
         },
+        has_not_key => [
+               'copyright Files' => qr/gone/,
+           ],
         file_check_sub => tweak(qw/log/),
     },
 
     {
         name => 'batmon-app',
         update => { in => path('t/scanner/examples/batmon.app.in'), quiet => 1 },
+        # test rename and read of debian/install file
         check => {
+            'install:"." content:0' => 'debian/batmon.desktop usr/share/applications',
+            'install:"." content:1' => 'batmon.xpm usr/share/pixmaps',
+            'install:./arm content:0' => '# dummy install for arm',
+            'install:batmon.app content:0' => '# dummy batmon.app.install',
+            'install:batmon.app/amd64 content:0' => '# dummy batmon.app.install for amd64',
         },
+        load => 'install:"." content:.push("plop.txt usr/share/doc/batmon.app")',
         # this entry is merged in debian/* *if* the Copyright coming from file is normalised
         # Also closes #862368
         has_not_key => [
@@ -294,6 +314,13 @@ sub tweak {
                 qr!Files: GNUmakefile!,
             ],
         },
+        file_contents_like => {
+            'debian/install' => [qr!batmon.desktop!, qr/plop/],
+        },
+        wr_check => {
+            'install:"." content:0' => 'debian/batmon.desktop usr/share/applications',
+            'install:"." content:1' => 'batmon.xpm usr/share/pixmaps',
+        },
     },
 
     {
@@ -319,33 +346,33 @@ sub tweak {
     },
 );
 
-my $cache_file = 't/model_tests.d/dependency-cache.txt';
+my $cache_file = path('t/model_tests.d/dependency-cache.txt');
 
 $Config::Model::Dpkg::Dependency::use_test_cache = 1;
+untie %Config::Model::Dpkg::Dependency::cache;
+%Config::Model::Dpkg::Dependency::cache = ();
 
-my $ch = new IO::File "$cache_file";
-foreach ( $ch->getlines ) {
-    chomp;
-    my ( $k, $v ) = split m/ => /;
+foreach my $line ( $cache_file->lines ) {
+    chomp $line;
+    next unless $line;
+    my ( $k, $v ) = split m/ => /, $line;
     $Config::Model::Dpkg::Dependency::cache{$k} = time . ' '. $v;
 }
-$ch->close;
 
 END {
     return if $::DebianDependencyCacheWritten;
     my %h = %Config::Model::Dpkg::Dependency::cache;
-    map { s/^\d+ //;} values %h ; # remove time stamp
+    do { s/^\d+ //;} for (values %h) ; # remove time stamp
     my $str = join( "\n", map { "$_ => $h{$_}"; } sort keys %h );
 
-    my $fh = new IO::File "> $cache_file";
     print "writing back cache file\n";
-    if ( defined $fh ) {
-
-        # not a big deal if cache cannot be written back
-        $fh->print($str);
-        $fh->close;
-        $::DebianDependencyCacheWritten = 1;
-    }
+    $cache_file->spew($str);
+    $::DebianDependencyCacheWritten = 1;
 }
 
-1;
+return {
+    conf_file_name => $conf_file_name,
+    conf_dir => $conf_dir,
+    home_for_test => $home_for_test,
+    tests => \@tests,
+};
